@@ -8,7 +8,7 @@ use Carp qw/carp croak/;
 use Scalar::Util qw/refaddr/;
 use List::Util qw/first/;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 my %header;
 
@@ -70,6 +70,11 @@ my %get = (
         $tags = join ' ', @{ $tags } if ref $tags eq 'ARRAY';
         $tags && qq{policyref="/w3c/p3p.xml", CP="$tags"};
     },
+    -server => sub {
+        my ( $header, $norm ) = @_;
+        return $ENV{SERVER_SOFTWARE} || 'cmdline' if $header->{-nph};
+        $header->{ $norm };
+    },
     -set_cookie    => sub { shift->{-cookie} },
     -window_target => sub { shift->{-target} },
 );
@@ -79,15 +84,11 @@ sub get {
     my $norm   = _normalize( shift ) || return;
     my $header = $header{ refaddr $self };
 
-    my $value;
     if ( my $get = $get{$norm} ) {
-        $value = $get->( $header, $norm );
+        return $get->( $header, $norm );
     }
-    else {
-        $value = $header->{ $norm };
-    }
-
-    $value;
+    
+    $header->{ $norm };
 }
 
 my %set = (
@@ -116,6 +117,7 @@ my %set = (
     -p3p => sub {
         carp "Can't assign to '-p3p' directly, use p3p_tags() instead";
     },
+    -server => sub {},
     -set_cookie => sub {
         my ( $header, $norm, $value ) = @_;
         delete $header->{-date} if $value;
@@ -130,14 +132,13 @@ my %set = (
 sub set {
     my $self   = shift;
     my $norm   = _normalize( shift ) || return;
-    my $value  = shift;
     my $header = $header{ refaddr $self };
 
     if ( my $set = $set{$norm} ) {
-        $set->( $header, $norm, $value );
+        $set->( $header, $norm, @_ );
     }
     else {
-        $header->{ $norm } = $value;
+        $header->{ $norm } = shift;
     }
 
     return;
@@ -157,6 +158,10 @@ my %exists = (
         exists $header->{ $norm }
             || first { $header->{$_} } qw(-nph -expires -cookie );
     },
+    -server => sub {
+        my ( $header, $norm ) = @_;
+        $header->{-nph} || exists $header->{ $norm };
+    },
     -set_cookie    => sub { exists shift->{-cookie} },
     -window_target => sub { exists shift->{-target} },
 );
@@ -166,15 +171,11 @@ sub exists {
     my $norm   = _normalize( shift ) || return;
     my $header = $header{ refaddr $self };
 
-    my $bool;
     if ( my $exists = $exists{$norm} ) {
-        $bool = $exists->( $header, $norm );
-    }
-    else {
-        $bool = exists $header->{ $norm };
+        return $exists->( $header, $norm );
     }
 
-    $bool;
+    exists $header->{ $norm };
 }
 
 my %delete = (
@@ -184,6 +185,10 @@ my %delete = (
         delete $header->{-charset};
         $header->{-type} = q{};
     },
+    -date    => sub {},
+    -expires => sub {},
+    -p3p     => sub {},
+    -server  => sub {},
     -set_cookie    => sub { delete shift->{-cookie} },
     -window_target => sub { delete shift->{-target} },
 );
@@ -192,16 +197,16 @@ sub delete {
     my $self   = shift;
     my $field  = shift;
     my $norm   = _normalize( $field ) || return;
-    my $value  = defined wantarray && $self->get( $field );
     my $header = $header{ refaddr $self };
 
     if ( my $delete = $delete{$norm} ) {
+        my $value  = defined wantarray && $self->get( $field );
         $delete->( $header, $norm );
+        delete $header->{ $norm };
+        return $value;
     }
 
     delete $header->{ $norm };
-
-    $value;
 }
 
 my %is_ignored = map { $_ => 1 }
@@ -273,27 +278,29 @@ sub p3p_tags {
 }
 
 sub field_names {
-    my $self   = shift;
-    my $header = $header{ refaddr $self };
-    my %header = %{ $header }; # copy
+    my $self    = shift;
+    my $header  = $header{ refaddr $self };
+    my %headers = %{ $header }; # copy
 
     my @fields;
 
-    push @fields, 'Status'        if delete $header{-status};
-    push @fields, 'Window-Target' if delete $header{-target};
-    push @fields, 'P3P'           if delete $header{-p3p};
+    push @fields, 'Server' if my $nph = delete $headers{-nph};
 
-    push @fields, 'Set-Cookie' if my $cookie  = delete $header{-cookie};
-    push @fields, 'Expires'    if my $expires = delete $header{-expires};
-    push @fields, 'Date' if delete $header{-nph} or $cookie or $expires;
+    push @fields, 'Status'        if delete $headers{-status};
+    push @fields, 'Window-Target' if delete $headers{-target};
+    push @fields, 'P3P'           if delete $headers{-p3p};
 
-    push @fields, 'Content-Disposition' if delete $header{-attachment};
+    push @fields, 'Set-Cookie' if my $cookie  = delete $headers{-cookie};
+    push @fields, 'Expires'    if my $expires = delete $headers{-expires};
+    push @fields, 'Date'       if $nph or $cookie or $expires;
 
-    my $type = delete @header{qw/-charset -type/};
+    push @fields, 'Content-Disposition' if delete $headers{-attachment};
+
+    my $type = delete @headers{qw/-charset -type/};
 
     # not ordered
-    while ( my ($norm, $value) = CORE::each %header ) {
-        next unless defined $value;
+    for my $norm ( keys %headers ) {
+        next unless defined $headers{ $norm };
 
         push @fields, do {
             my $field = $norm;
@@ -344,14 +351,14 @@ sub as_string {
 
     my @lines;
 
+    # add Status-Line
     if ( $header->{-nph} ) {
-        my $software = $ENV{SERVER_SOFTWARE} || 'cmdline';
         my $protocol = $ENV{SERVER_PROTOCOL} || 'HTTP/1.0';
         my $status   = $header->{-status}    || '200 OK';
         push @lines, "$protocol $status";
-        push @lines, "Server: $software";
     }
 
+    # add response headers
     $self->each(sub {
         my ( $field, $value ) = @_;
         $value =~ s/$eol(\s)/$1/g;
@@ -443,8 +450,8 @@ CGI::Header - Adapter for CGI::header() function
 
 =head1 DESCRIPTION
 
-Utility class to manipulate a hash reference which L<CGI>'s C<header()>
-function receives.
+This module is a utility class to manipulate a hash reference
+which L<CGI>'s C<header()> function receives.
 
 =head2 METHODS
 
@@ -497,7 +504,8 @@ Returns the value of the deleted field.
 
 =item @fields = $header->field_names
 
-Returns the list of field names present in the header.
+Returns the list of distinct field names present in the header.
+The field names have case as returned by C<CGI::header()>.
 
   my @fields = $header->field_names;
   # => ( 'Set-Cookie', 'Content-length', 'Content-Type' )
@@ -507,6 +515,8 @@ Returns the list of field names present in the header.
 Apply a subroutine to each header field in turn.
 The callback routine is called with two parameters;
 the name of the field and a value.
+If the Set-Cookie header is multi-valued, then the routine is called
+once for each value.
 Any return values of the callback routine are ignored.
 
   my @lines;
@@ -518,10 +528,14 @@ Any return values of the callback routine are ignored.
 
 =item @headers = $header->flatten
 
-Returns pairs of fields and values.
+Returns pairs of fields and values. It's identical to:
 
-  my @headers = $header->flatten;
-  # => ( 'Status', '304 Nod Modified', 'Content-Type', 'text/plain' )
+  my @headers;
+
+  $header->each(sub {
+      my ( $field, $value ) = @_;
+      push @headers, $field, $value;
+  });
 
 =item $header->clear
 
@@ -567,23 +581,42 @@ is identical to:
   my $CRLF = $CGI::CRLF;
   print $header->as_string( $CRLF ), $CRLF;
 
+When valid multi-line headers are included, this method will always output
+them back as a single line, according to the folding rules of RFC 2616:
+the newlines will be removed, while the white space remains.
+
+Unlike CGI.pm, when invalid newlines are included,
+this module removes them instead of throwing exceptions.
+
+=item $filename = $header->attachment
+
 =item $header->attachment( $filename )
 
-A shortcut for
+Can be used to turn tha page into an attachment.
+Represents suggested name for the saved file.
 
-  $header->set(
-      'Content-Disposition' => qq{attachment; filename="$filename"}
-  );
+  $header->attachment( 'genome.jpg' );
 
-=item $header->p3p_tags( $tags )
+In this case, the outgoing will be formatted as:
 
-A shortcut for
+  Content-Disposition: attachment; filename="genome.jpg"
 
-  $header->set(
-      'P3P' => qq{policyref="/w3c/p3p.xml", CP="$tags"}
-  ); 
+=item @tags = $header->p3p_tags
 
-=item $header->expires
+=item $header->p3p_tags( @tags )
+
+Represents P3P tags. The parameter can be an array or a space-delimited
+string. Returns a list of P3P tags.
+
+  $header->p3p_tags(qw/CAO DSP LAW CURa/);
+
+In this case, the outgoing header will be formatted as:
+
+  P3P: policyref="/w3c/p3p.xml", CP="CAO DSP LAW CURa"
+
+=item $format = $header->expires
+
+=item $header->expires( $format )
 
 The Expires header gives the date and time after which the entity
 should be considered stale. You can specify an absolute or relative
@@ -599,6 +632,13 @@ expiration interval. The following forms are all valid for this field:
   # at the indicated time & date
   $header->expires( 'Thu, 25 Apr 1999 00:40:33 GMT' );
 
+=item $header->nph
+
+If set to a true value, will issue the correct headers to work
+with a NPH (no-parse-header) script.
+
+  $header->nph( 1 );
+
 =back
 
 =head1 SEE ALSO
